/**********************************************************************
file: xadc_arty_core.h - definition of XadcArtyCore class
Copyright (C) 2017  p. chu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
**********************************************************************/


/*****************************************************************//**
 * @file xadc_arty_core.h
 *
 * @brief retrieve data from Arty board's XADC core
 *
 * @note XadcArtyCore: customized for Arty A7 board xadc configuration;
 *
 *
 * @author p chu
 * @version v1.0: initial release
 *********************************************************************/

#ifndef _XADC_ARTY_CORE_H_INCLUDED
#define _XADC_ARTY_CORE_H_INCLUDED

#include "chu_init.h"

/*********************************************************************
 * channel mapping (arty signal: aux)
 * single-ended channels (max=3V via voltage divider)
 * a0: 4
 * a1: 5
 * a2: 6
 * a3: 7
 * a4: 15
 * a5: 0
 * differential channels  (max=1V)
 * a6/a7: 12 (diff)
 * a8/a9: 13 (diff)
 * a10/a11: 14 (diff)
 * on-board v-i channel
 *   - iv0 (core current): 10 (max=1V)
 *       shunt register = 10m ohm; amp gain = 50
 *       1A => 0.5V
 *   - iv1 (unregulated voltage): 2 (max=16V via voltage divider)
 *   - iv2 (5V power supply voltage): 1 (max=5.99V via voltage divider)
 *   - iv3 (5V power supply current): 9
 *       shunt register = 5m ohm; amp gain = 50
 *       1A => 0.25V
 * software driver must do the conversion
 *********************************************************************
 * channel mapping (aux: arty signal)
 * 0:a5   1:iv2  2:iv1    3:n/a  4:a0     5:a1     6:a2       7:a3
 * 8:n/a  9:iv3  10:iv0  11:n/a 12:a6/a7 13:a8/a9 14:a10/a11 15:a4
 *********************************************************************/

/**
 * adsr core driver:
 * - retrieve data from 6 xadc channels
 */
class XadcArtyCore {
public:
	/**
	 * register map
	 */
	enum {
		TMP_REG = 0x00, /**< FPGA internal temperature */
		CORE_V_REG = 0x01, /**< FPGA internal core voltage */
		VP_VN_REG = 0x03, /**< dedicated vp/vn analog input */
		CORE_I_REG = 0x1a, /**< core current */
		AUX_0_REG = 0x10, /**< 16-bit data from Arty xadc aux channel #0 */
		RAW_V_REG = 0x12, /**< unregulated raw voltage from power source */
		V5_V_REG = 0x11, /**< voltage from 5V regulator  */
		V5_I_REG = 0x19, /**< current from 5V regulator  */
	};

	/**
	 * constructor.
	 */
	XadcArtyCore(uint32_t core_base_addr);
	~XadcArtyCore(); // not used

	/**
	 * retrieve raw xadc data
	 *
	 * @param n xadc aux channel
	 * @return raw 16-bit data
	 * @note only 12 MSBs is used for adc data
	 */
	uint16_t read_channel_raw(int n);

	/**
	 * retrieve raw xadc voltage
	 *
	 * @param n xadc aux channel
	 * @return raw voltage (0.0V to 1.0V)
	 */
	double read_channel_voltage(int n);

	/**
	 * retrieve arty analog inputs
	 *
	 * @param n analog input number (0 - 5, 6, 8, 10)
	 * @return voltage
	 * @note analog input 0 to 5: return 0 to 3.3V (due to voltage divider)
	 * @note analog input 6, 8, 10: return 0 to 1.0V
	 */
	double read_adc_in(int n);

	/**
	 * retrieve FPGA internal vcc
	 * @return FPGA core Vcc (about 1.0V)
	 * @note vcc=3*(adc reading)
	 */
	double read_fpga_vcc();

	/**
	 * retrieve vp/vn input voltage
	 * @return vp/vn voltage (0 to 1.0V)
	 */
	double read_vp_vn();

	/**
	 * retrieve FPGA internal temperature
	 * @return FPGA core temperature in Celsius
	 * @note see Xilinx ug480
	 */
	double read_fpga_temp();

	/**
	 * retrieve core current supplied to FPGA
	 * @return FPGA core current in Amp
	 */
	double read_fpga_icc();

	/**
	 * retrieve voltage supplied to the board
	 * @return voltage in Volt
	 */
	double read_board_v();

	/**
	 * retrieve voltage from board's 5V regulator
	 * @return voltage in Volt
	 * @note should be around 5V
	 */
	double read_5v_v();

	/**
	 * retrieve current from board's 5V regulator
	 * @return current in amp
	 */
	double read_5v_i();

private:
	/* variable to keep track of current status */
	uint32_t base_addr;
	/* arty xadc channel map: A# of arduino analog input => xadc aux channel */
	const int CHN_MAP[12] = { 4, 5, 6, 7, 15, 0, 12, 12, 13, 13, 14, 14 };
}
;

#endif  // _XADC_ARTY_CORE_H_INCLUDED
