library ieee;
use ieee.std_logic_1164.all;
use work.chu_io_map.all;
entity mcs_top_sampler_arty_a7 is
   generic(BRIDGE_BASE : std_logic_vector(31 downto 0) := x"C0000000");
   port(
      clk        : in    std_logic;
      reset_n    : in    std_logic;
      -- switches and LEDs
      sw         : in    std_logic_vector(3 downto 0);
      led        : out   std_logic_vector(3 downto 0);
      -- uart
      rx         : in    std_logic;
      tx         : out   std_logic;
      -- 
      -- arty xadc analog channels 
      vp_in, vn_in : in  std_logic;
      adc_a_p      : in    std_logic_vector(8 downto 0);
      adc_a_n      : in    std_logic_vector(8 downto 0);
      adc_iv_p     : in    std_logic_vector(3 downto 0);
      adc_iv_n     : in    std_logic_vector(3 downto 0);
      -- rgb leds
      rgb_led0   : out   std_logic_vector(2 downto 0);
      rgb_led1   : out   std_logic_vector(2 downto 0);
      rgb_led2   : out   std_logic_vector(2 downto 0);
      rgb_led3   : out   std_logic_vector(2 downto 0);
      -- buttons
      btn        : in    std_logic_vector(3 downto 0);
      -- arduino i/o
      arduino_io : inout std_logic_vector(15 downto 0);
      -- PMOD JA (divided into top row and bottom row
      ja_top     : out   std_logic_vector(4 downto 1);
      ja_btm     : out   std_logic_vector(10 downto 7)
   );
end mcs_top_sampler_arty_a7;

architecture arch of mcs_top_sampler_arty_a7 is
   component cpu
      port(
         clk             : in  std_logic;
         reset           : in  std_logic;
         io_addr_strobe  : out std_logic;
         io_read_strobe  : out std_logic;
         io_write_strobe : out std_logic;
         io_address      : out std_logic_vector(31 downto 0);
         io_byte_enable  : out std_logic_vector(3 downto 0);
         io_write_data   : out std_logic_vector(31 downto 0);
         io_read_data    : in  std_logic_vector(31 downto 0);
         io_ready        : in  std_logic
      );
   end component;
   signal io_addr_strobe  : std_logic;
   signal io_read_strobe  : std_logic;
   signal io_write_strobe : std_logic;
   signal io_byte_enable  : std_logic_vector(3 downto 0);
   signal io_address      : std_logic_vector(31 downto 0);
   signal io_write_data   : std_logic_vector(31 downto 0);
   signal io_read_data    : std_logic_vector(31 downto 0);
   signal io_ready        : std_logic;
   signal mmio_cs         : std_logic;
   signal mmio_wr         : std_logic;
   signal mmio_rd         : std_logic;
   signal mmio_addr       : std_logic_vector(20 downto 0);
   signal mmio_wr_data    : std_logic_vector(31 downto 0);
   signal mmio_rd_data    : std_logic_vector(31 downto 0);
   -- clk/reset related
   signal clk_100M        : std_logic;
   signal reset_sys       : std_logic;
   -- pwm 
   signal pwm             : std_logic_vector(15 downto 0);
   -- ddfs/audio pdm 
   signal pdm             : std_logic;
   signal ddfs_sq_wave    : std_logic;
   -- tft spi 
   signal sclk            : std_logic;
   signal mosi            : std_logic;
   signal miso            : std_logic;
   signal ss_n            : std_logic_vector(3 downto 0);

begin
   -- clock and reset
   clk_100M               <= clk;      -- 100 MHz external clock
   reset_sys              <= not reset_n;
   --  rgb leds
   rgb_led0               <= pwm(2 downto 0);
   rgb_led1               <= pwm(5 downto 3);
   rgb_led2               <= pwm(8 downto 6);
   rgb_led3               <= pwm(11 downto 9);
   arduino_io(2)          <= ddfs_sq_wave;
   arduino_io(3)          <= pdm;
   arduino_io(7 downto 4) <= pwm(15 downto 12);
   arduino_io(8)          <= ss_n(2);
   arduino_io(9)          <= ss_n(1);
   arduino_io(10)         <= ss_n(0);
   arduino_io(11)         <= mosi;
   miso                   <= arduino_io(12);
   arduino_io(13)         <= sclk;

   -- PMOD JA bottom for Pmod AMP2  
   ja_btm(7) <= pdm; -- pdm audio
   ja_btm(8) <= '0'; -- 1/0 for 6/12db gain 
   ja_btm(9) <= '0'; -- not used
   ja_btm(10) <= '1'; -- not shut down
   
   ja_top       <= "0000";
   
   -- instantiate microBlaze MCS
   mcs_0 : cpu
      port map(
         clk             => clk_100M,
         reset           => reset_sys,
         io_addr_strobe  => io_addr_strobe,
         io_read_strobe  => io_read_strobe,
         io_write_strobe => io_write_strobe,
         io_byte_enable  => io_byte_enable,
         io_address      => io_address,
         io_write_data   => io_write_data,
         io_read_data    => io_read_data,
         io_ready        => io_ready
      );
   -- instantiate MCS IO bus to FPro bus bridge
   bridge_unit : entity work.chu_mcs_bridge
      generic map(BRG_BASE => BRIDGE_BASE)
      port map(
         io_addr_strobe  => io_addr_strobe,
         io_read_strobe  => io_read_strobe,
         io_write_strobe => io_write_strobe,
         io_byte_enable  => io_byte_enable,
         io_address      => io_address,
         io_write_data   => io_write_data,
         io_read_data    => io_read_data,
         io_ready        => io_ready,
         fp_video_cs     => open,
         fp_mmio_cs      => mmio_cs,
         fp_wr           => mmio_wr,
         fp_rd           => mmio_rd,
         fp_addr         => mmio_addr,
         fp_wr_data      => mmio_wr_data,
         fp_rd_data      => mmio_rd_data
      );
   -- instantiate sampler MMIO subsystem 
   mmio_sys_unit : entity work.mmio_sys_sampler_arty_a7
      port map(
         clk          => clk_100M,
         reset        => reset_sys,
         mmio_cs      => mmio_cs,
         mmio_wr      => mmio_wr,
         mmio_rd      => mmio_rd,
         mmio_addr    => mmio_addr,
         mmio_wr_data => mmio_wr_data,
         mmio_rd_data => mmio_rd_data,
         sw           => sw,
         led          => led,
         rx           => rx,
         tx           => tx,
         vp_in       => vp_in,
         vn_in       => vn_in,
         adc_a_p     => adc_a_p,
         adc_a_n     => adc_a_n,
         adc_iv_p     => adc_iv_p,
         adc_iv_n     => adc_iv_n,
         pwm          => pwm,
         btn          => btn,
         an           => open,       --an,
         sseg         => open,       --sseg,
         sclk         => sclk,
         mosi         => mosi,
         miso         => miso,
         ss_n         => ss_n,
         i2c_scl      => arduino_io(15),
         i2c_sda      => arduino_io(14),
         ps2d         => open,
         ps2c         => open,
         ddfs_sq_wave => ddfs_sq_wave,
         pdm          => pdm
      );

end arch;

