--*********************************************************************
--file: chu_xadc_arty_a7_core.vhd - xadc core with 16 aux channels 
--Copyright (C) 2017  p. chu
--
--This program is free software: you can redistribute it and/or modify
--it under the terms of the GNU General Public License as published by
--the Free Software Foundation, either version 3 of the License, or
--(at your option) any later version.
--
--This program is distributed in the hope that it will be useful,
--but WITHOUT ANY WARRANTY; without even the implied warranty of
--MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--GNU General Public License for more details.
--
--You should have received a copy of the GNU General Public License
--along with this program.  If not, see <https://www.gnu.org/licenses/>.
--*********************************************************************


--*********************************************************************
-- Xilinx xadc interface:
--  * xadc in sequence mode
--  * DRP interface is connected to atomtically read
--    out the pres-designated channels
--  * the readout is stored into corresponding register
--*********************************************************************
-- arty board configuration
--  * vp/vn channel
--  * 13 aux channels:
--      * 6 single-ended channels
--      * 3 differential channels
--      * 4 on-board voltage/current channels
--  * the design enables all 16 aux channels and 
--    * 16 aux reading in a register file
--    * vp/temp/vcc in three registers 
-- *********************************************************************
--  * channel mapping (arty signal: aux)
--    * single-ended channels (max=3V via voltage divider)
--      * a0: 4
--      * a1: 5
--      * a2: 6
--      * a3: 7
--      * a4: 15
--      * a5: 0
--    * differential channels  (max=1V)
--      * a6/a7: 12 (diff)
--      * a8/a9: 13 (diff)
--      * a10/a11: 14 (diff)
--    * on-board v-i channel
--      * iv0 (core current): 10 (max=1V)
--          * shunt register = 10m ohm; amp gain = 50 
--          * 1A => 0.5V
--      * iv1 (unregulated voltage): 2 (max=16V via volatge divider)
--          * voltage of external power jack (usually not connected) 
--      * iv2 (5V power supply voltage): 1 (max=5.99V via voltage divider) 
--      * iv3 (5V power supply current): 9 
--          * shunt register = 5m ohm; amp gain = 50 
--          * 1A => 0.25V
--    * software driver must do the conversion  
--*********************************************************************
--  * channel mapping (aux: arty signal)
--    * 0:a5   1:iv2  2:iv1    3:n/a  4:a0     5:a1     6:a2       7:a3
--    * 8:n/a  9:iv3  10:iv0  11:n/a 12:a6/a7 13:a8/a9 14:a10/a11 15:a4   
--*********************************************************************
--*********************************************************************
--  * xadc internal reg address space (5 bits) 
--    * 0xxxx: internal measurements and vp/vn
--    * 1xxxx: 16 aux analog readings    
--*********************************************************************
--*********************************************************************
--  * xadc_arty_core MMIO reg map 
--    * 1xxxx: 16 aux analog readings    
--    * 00000: FPGA die temperature
--    * 00001: FPGA core voltage 
--    * 00011: vp/vn
--*********************************************************************
library ieee;
use ieee.std_logic_1164.all;
entity chu_xadc_arty_a7_core is
   port(
      clk     : in  std_logic;
      reset   : in  std_logic;
      -- slot interface
      cs      : in  std_logic;
      write   : in  std_logic;
      read    : in  std_logic;         
      addr    : in  std_logic_vector(4 downto 0);  
      rd_data : out std_logic_vector(31 downto 0);
      wr_data : in  std_logic_vector(31 downto 0);
      -- external signals 
      vp_in, vn_in : in  std_logic;
      adc_a_p   : in  std_logic_vector(8 downto 0); -- 9 analog channels
      adc_a_n   : in  std_logic_vector(8 downto 0);
      adc_iv_p  : in  std_logic_vector(3 downto 0); -- 4 i-v channels
      adc_iv_n  : in  std_logic_vector(3 downto 0)
   );
end chu_xadc_arty_a7_core;

architecture arch of chu_xadc_arty_a7_core is
   component xadc_arty_a7_fpro
     port (
       di_in : in std_logic_vector(15 downto 0);
       daddr_in : in std_logic_vector(6 downto 0);
       den_in : in std_logic;
       dwe_in : in std_logic;
       drdy_out : out std_logic;
       do_out : out std_logic_vector(15 downto 0);
       dclk_in : in std_logic;
       reset_in : in std_logic;
       channel_out : out std_logic_vector(4 downto 0);
       eoc_out : out std_logic;
       alarm_out : out std_logic;
       eos_out : out std_logic;
       busy_out : out std_logic;
       ot_out: out std_logic;
       user_temp_alarm_out: out std_logic;
       vp_in : in std_logic;
       vn_in : in std_logic;
       vauxp0 : in std_logic;
       vauxn0 : in std_logic;
       vauxp1 : in std_logic;
       vauxn1 : in std_logic;
       vauxp2 : in std_logic;
       vauxn2 : in std_logic;
       vauxp3 : in std_logic;
       vauxn3 : in std_logic;
       vauxp4 : in std_logic;
       vauxn4 : in std_logic;
       vauxp5 : in std_logic;
       vauxn5 : in std_logic;
       vauxp6 : in std_logic;
       vauxn6 : in std_logic;
       vauxp7 : in std_logic;
       vauxn7 : in std_logic;
       vauxp8 : in std_logic;
       vauxn8 : in std_logic;
       vauxp9 : in std_logic;
       vauxn9 : in std_logic;
       vauxp10 : in std_logic;
       vauxn10 : in std_logic;
       vauxp11 : in std_logic;
       vauxn11 : in std_logic;
       vauxp12 : in std_logic;
       vauxn12 : in std_logic;
       vauxp13 : in std_logic;
       vauxn13 : in std_logic;
       vauxp14 : in std_logic;
       vauxn14 : in std_logic;
       vauxp15 : in std_logic;
       vauxn15 : in std_logic
    );
   end component;
   --signal vn_in, vp_in : std_logic;
   signal channel      : std_logic_vector(4 downto 0);
   signal daddr_in     : std_logic_vector(6 downto 0);
   signal eoc          : std_logic;
   signal rdy          : std_logic;
   signal wr_fifo      : std_logic;
   signal fifo_readout: std_logic_vector(15 downto 0);
   signal adc_data     : std_logic_vector(15 downto 0);
   signal tmp_out_reg  : std_logic_vector(15 downto 0);
   signal vcc_out_reg  : std_logic_vector(15 downto 0);
   signal vpn_out_reg  : std_logic_vector(15 downto 0);
begin
   -- instantiate cusomized xadc core  
   xdac_unit : xadc_arty_a7_fpro
      port map(
         dclk_in     => clk,
         reset_in    => reset,          --reset,
         di_in       => (others => '0'),
         daddr_in    => daddr_in,
         den_in      => eoc,
         dwe_in      => '0',            -- read only
         drdy_out    => rdy,
         do_out      => adc_data,
         channel_out => channel,
         eoc_out     => eoc,
         eos_out     => open,
         busy_out    => open,
         alarm_out   => open,
         ot_out  => open,
         user_temp_alarm_out => open,
         vp_in   => vp_in, 
         vn_in   => vn_in, 
         vauxp0  => adc_a_p(5), 
         vauxn0  => adc_a_n(5),
         vauxp1  => adc_iv_p(2),
         vauxn1  => adc_iv_n(2),
         vauxp2  => adc_iv_p(1),
         vauxn2  => adc_iv_n(1),
         vauxp3  => '0',
         vauxn3  => '0',
         vauxp4  => adc_a_p(0),
         vauxn4  => adc_a_n(0),
         vauxp5  => adc_a_p(1),
         vauxn5  => adc_a_n(1),
         vauxp6  => adc_a_p(2),
         vauxn6  => adc_a_n(2),
         vauxp7  => adc_a_p(3),
         vauxn7  => adc_a_n(3),
         vauxp8  => '0',
         vauxn8  => '0',
         vauxp9  => adc_iv_p(3),
         vauxn9  => adc_iv_n(3),
         vauxp10 => adc_iv_p(0),
         vauxn10 => adc_iv_n(0),
         vauxp11 => '0',
         vauxn11 => '0',
         vauxp12 => adc_a_p(6),
         vauxn12 => adc_a_n(6),
         vauxp13 => adc_a_p(7),
         vauxn13 => adc_a_n(7),
         vauxp14 => adc_a_p(8),
         vauxn14 => adc_a_n(8),
         vauxp15 => adc_a_p(4),
         vauxn15 => adc_a_n(4)
      );
      
   -- form xadc DRP address 
   daddr_in <= "00" & channel;
   
   -- register file 
   reg_file_unit : entity work.reg_file(arch)
      generic map(
         DATA_WIDTH => 16,
         ADDR_WIDTH => 4
      )
      port map(
         clk    => clk,
         w_addr => channel(3 downto 0),
         w_data => adc_data,
         r_addr => addr(3 downto 0),
         r_data => fifo_readout,
         wr_en  => wr_fifo
      );
    -- aux register addr is "1xxxx" 
    wr_fifo <= '1' when rdy = '1' and channel(4)='1' else '0';    

   -- registers and decoding 
  process(clk, reset)
   begin
      if reset = '1' then
         tmp_out_reg  <= (others => '0');
         vcc_out_reg  <= (others => '0');
         vpn_out_reg <= (others => '0');
      elsif (clk'event and clk = '1') then
         if rdy = '1' and channel = "00000" then
            tmp_out_reg <= adc_data;
         end if;
         if rdy = '1' and channel = "00001" then
            vcc_out_reg <= adc_data;
         end if;
         if rdy = '1' and channel = "00011" then
            vpn_out_reg <= adc_data;
         end if;
      end if;
   end process;
   -- read multiplexing   
   rd_data <= 
      x"0000" & fifo_readout when addr(4)='1' else
      x"0000" & tmp_out_reg  when addr="00000" else
      x"0000" & vpn_out_reg  when addr="00011" else
      x"0000" & vcc_out_reg;
end arch;

